// Car -- a simple Car class that stores information
// about a rectangle (simulating a car in Rush Hour)
// with a colour.  It also knows how to calculate
// the legal distance a car can travel before it bumps
// into other cars.

// A small helper function to check if the number
// range from a to b overlaps the range of c to d.
// You don't ever need to use this directly.
boolean checkOverlap( int a, int b, int c, int d )
{
  return (a < d) && (b > c);
}

class Car
{
  int x;
  int y;
  int w;
  int h;
  color col;

  Car( int x, int y, int w, int h )
  {
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
    col = color( #a0a0a0 );
  }

  void setColour( color c )
  {
    col = c;
  }

  // This this Car to draw itself.
  void draw()
  {
    fill( col );
    rect( x, y, w, h );
  }

  // Decide whether a Car is oriented horizontally
  // or vertically.  You don't ever need to call this
  // yourself.
  boolean isHorizontal()
  {
    return w > h;
  }

  // Given a location (px,py) in the sketch window, 
  // decide whether that location touches this Car.
  // Useful in a direct manipulation interface.
  boolean hitTest( int px, int py )
  {
    return (px >= x) && (px < (x+w)) && 
      (py >= y) && (py < (y + h));
  }

  // Try to move this Car by distance dx in the x direction
  // and dy in the y direction (relative to the Car's
  // current position).  Also pass in an array of all Cars
  // in the world.  If the movement would cause a collision,
  // stop the movement before that happens.  Force Cars
  // to move horizontally or vertically.
  void move( int dx, int dy, Car[] blockers )
  {
    // There are a few complexities in getting this to
    // work, and you're not required to understand the
    // internals.  If you want to dive in a bit, note
    // that the horizontal and vertical cases are 
    // almost identical to each other.
    if ( isHorizontal() ) {
      int xmin = -1000;
      int xmax = 1000;

      for ( int idx = 0; idx < blockers.length; ++idx ) {
        Car other = blockers[idx];
        if ( other == this ) {
          // Don't check for a collision against yourself.
          continue;
        }
        if ( checkOverlap( y, y + h, other.y, other.y + other.h ) ) {
          if ( (other.x + other.w) <= x ) {
            xmin = max( xmin, other.x + other.w );
          }
          if ( (x + w) <= other.x ) {
            xmax = min( xmax, other.x );
          }
        }
      }

      x = constrain( x + dx, xmin, xmax - w );
    } else {
      int ymin = -1000;
      int ymax = 1000;

      for ( int idx = 0; idx < blockers.length; ++idx ) {
        Car other = blockers[idx];
        if ( other == this ) {
          continue;
        }
        if ( checkOverlap( x, x + w, other.x, other.x + other.w ) ) {
          if ( (other.y + other.h) <= y ) {
            ymin = max( ymin, other.y + other.h );
          }
          if ( (y + h) <= other.y ) {
            ymax = min( ymax, other.y );
          }
        }
      }

      y = constrain( y + dy, ymin, ymax - h );
    }
  }
}